WRITE MARKDOWN WITH 8 EXCEPTIONAL OPEN SOURCE EDITORS
============================================================

### Markdown

By way of a succinct introduction, Markdown is a lightweight plain text formatting syntax created by John Gruber together with Aaron Swartz. Markdown offers individuals “to write using an easy-to-read, easy-to-write plain text format, then convert it to structurally valid XHTML (or HTML)”. Markdown’s syntax consists of easy to remember symbols. It has a gentle learning curve; you can literally learn the Markdown syntax in the time it takes to fry some mushrooms (that’s about 10 minutes). By keeping the syntax as simple as possible, the risk of errors is minimized. Besides being a friendly syntax, it has the virtue of producing clean and valid (X)HTML output. If you have seen my HTML, you would know that’s pretty essential.

The main goal for the formatting syntax is to make it extremely readable. Users should be able to publish a Markdown-formatted document as plain text. Text written in Markdown has the virtue of being easy to share between computers, smart phones, and individuals. Almost all content management systems support Markdown. It’s popularity as a format for writing for the web has also led to variants being adopted by many services such as GitHub and Stack Exchange.

Markdown can be composed in any text editor. But I recommend an editor purposely designed for this syntax. The software featured in this roundup allows an author to write professional documents of various formats including blog posts, presentations, reports, email, slides and more. All of the applications are, of course, released under an open source license. Linux, OS X and Windows’ users are catered for.

* * *

### Remarkable

 ![Remarkable - cross-platform Markdown editor](https://i2.wp.com/www.ossblog.org/wp-content/uploads/2017/02/Remarkable.png?resize=800%2C319&ssl=1) 

Let’s start with Remarkable. An apt name. Remarkable is a reasonably featured Markdown editor – it doesn’t have all the bells and whistles, but there’s nothing critical missing. It has a syntax like Github flavoured markdown.

With this editor you can write Markdown and view the changes as you make them in the live preview window. You can export your files to PDF (with a TOC) and HTML. There are multiple styles available along with extensive configuration options so you can configure it to your heart’s content.

Other features include:

*   Syntax highlighting
*   GitHub Flavored Markdown support
*   MathJax support – render rich documents with advanced formatting
*   Keyboard shortcuts

There are easy installers available for Debian, Ubuntu, Fedora, SUSE and Arch systems.

Homepage: [https://remarkableapp.github.io/][4]
License: MIT License

* * *

### Atom

 ![Atom - cross-platform Markdown editor](https://i2.wp.com/www.ossblog.org/wp-content/uploads/2017/02/Atom-Markdown.png?resize=800%2C328&ssl=1) 

Make no bones about it, Atom is a fabulous text editor. Atom consists of over 50 open source packages integrated around a minimal core. With Node.js support, and a full set of features, Atom is my preferred way to edit code. It features in our [Killer Open Source Apps][5], it is that masterly. But as a Markdown editor Atom leaves a lot to be desired – its default packages are bereft of Markdown specific features; for example, it doesn’t render equations, as illustrated in the graphic above.

But here lies the power of open source and one of the reasons I’m a strong advocate of openness. There are a plethora of packages, some forks, which add the missing functionality. For example, Markdown Preview Plus provides a real-time preview of markdown documents, with math rendering and live reloading. Alternatively, you might try [Markdown Preview Enhanced][6]. If you need an auto-scroll feature, there’s [markdown-scroll-sync][7]. I’m a big fan of [Markdown-Writer][8] and [markdown-pdf][9] the latter converts markdown to PDF, PNG and JPEG on the fly.

The approach embodies the open source mentality, allowing the user to add extensions to provide only the features needed. Reminds me of Woolworths pick ‘n’ mix sweets. A bit more effort, but the best outcome.

Homepage: [https://atom.io/][10]
License: MIT License

* * *

### Haroopad

 ![Haroopad - - cross-platform Markdown editor](https://i2.wp.com/www.ossblog.org/wp-content/uploads/2017/02/Haroopad-1.png?resize=800%2C332&ssl=1) 

Haroopad is an excellent markdown enabled document processor for creating web-friendly documents. Author various formats of documents such as blog articles, slides, presentations, reports, and e-mail. Haroopad runs on Windows, Mac OS X, and Linux. There are Debian/Ubuntu packages, and binaries for Windows and Mac. The application uses node-webkit, CodeMirror, marked, and Twitter Bootstrap.

Haroo means “A Day” in Korean.

The feature list is rather impressive; take a look below:

*   Themes, Skins and UI Components
    *   Over 30 different themes to edit – tomorrow-night-bright and zenburn are recent additions
    *   Syntax highlighting in fenced code block on editor
    *   Ruby, Python, PHP, Javascript, C, HTML, CSS
    *   Based on CodeMirror, a versatile text editor implemented in JavaScript for the browser
*   Live Preview themes
    *   7 themes based markdown-css
*   Syntax Highlighting
    *   112 languages & 49 styles based on highlight.js
*   Custom Theme
    *   Style based on CSS (Cascading Style Sheet)
*   Presentation Mode – useful for on the spot presentations
*   Draw diagrams – flowcharts, and sequence diagrams
*   Tasklist
*   Enhanced Markdown syntax with TOC, GitHub Flavored Markdown and extensions, mathematical expressions, footnotes, tasklists, and more
*   Font Size
    *   Editor and Viewer font size control using Preference Window & Shortcuts
*   Embedding Rich Media Contents
    *   Video, Audio, 3D, Text, Open Graph and oEmbed
    *   About 100 major internet services (YouTube, SoundCloud, Flickr …) Support
    *   Drag & Drop support
*   Display Mode
    *   Default (Editor:Viewer), Reverse (Viewer:Editor), Only Editor, Only Viewer (View > Mode)
*   Insert Current Date & Time
    *   Various Format support (Insert > Date & Time)
*   HTML to Markdown
    *   Drag & Drop your selected text on Web Browser
*   Options for markdown parsing
*   Outline View
*   Vim Key-binding for purists
*   Markdown Auto Completion
*   Export to PDF, HTML
*   Styled HTML copy to clipboard for WYSIWYG editors
*   Auto Save & Restore
*   Document state information
*   Tab or Spaces for Indentation
*   Column (Single, Two and Three) Layout View
*   Markdown Syntax Help Dialog.
*   Import and Export settings
*   Support for LaTex mathematical expressions using MathJax
*   Export documents to HTML and PDF
*   Build extensions for making your own feature
*   Effortlessly transform documents into a blog system: WordPress, Evernote and Tumblr,
*   Full screen mode – although the mode fails to hide the top menu bar or the bottom toolbar
*   Internationalization support: English, Korean, Spanish, Chinese Simplified, German, Vietnamese, Russian, Greek, Portuguese, Japanese, Italian, Indonesian, Turkish, and French

Homepage: [http://pad.haroopress.com/][11]
License: GNU GPL v3

* * *

### StackEdit

 ![StackEdit - a web based Markdown editor](https://i2.wp.com/www.ossblog.org/wp-content/uploads/2017/02/StackEdit.png?resize=800%2C311&ssl=1) 

StackEdit is a full-featured Markdown editor based on PageDown, the Markdown library used by Stack Overflow and the other Stack Exchange sites. Unlike the other editors in this roundup, StackEdit is a web based editor. A Chrome app is also available.

Features include:

*   Real-time HTML preview with Scroll Link feature to bind editor and preview scrollbars
*   Markdown Extra/GitHub Flavored Markdown support and Prettify/Highlight.js syntax highlighting
*   LaTeX mathematical expressions using MathJax
*   WYSIWYG control buttons
*   Configurable layout
*   Theming support with different themes available
*   A la carte extensions
*   Offline editing
*   Online synchronization with Google Drive (multi-accounts) and Dropbox
*   One click publish on Blogger, Dropbox, Gist, GitHub, Google Drive, SSH server, Tumblr, and WordPress

Homepage: [https://stackedit.io/][12]
License: Apache License

* * *

### MacDown

 ![MacDown - OS X Markdown editor](https://i0.wp.com/www.ossblog.org/wp-content/uploads/2017/02/MacDown.png?resize=800%2C422&ssl=1) 

MacDown is the only editor featured in this roundup which only runs on macOS. Specifically, it requires OS X 10.8 or later. Hoedown is used internally to render Markdown into HTML which gives an edge to its performance. Hoedown is a revived fork of Sundown, it is fully standards compliant with no dependencies, good extension support, and UTF-8 aware.

MacDown is based on Mou, a proprietary solution designed for web developers.

It offers good Markdown rendering, syntax highlighting for fenced code blocks with language identifiers rendered by Prism, MathML and LaTeX rendering, GTM task lists, Jekyll front-matter, and optional advanced auto-completion. And above all, it isn’t a resource hog. Want to write Markdown on OS X? MacDown is my open source recommendation for web developers.

Homepage: [https://macdown.uranusjr.com/][13]
License: MIT License

* * *

### ghostwriter

 ![ghostwriter - cross-platform Markdown editor](https://i0.wp.com/www.ossblog.org/wp-content/uploads/2017/02/ghostwriter.png?resize=800%2C310&ssl=1) 

ghostwriter is a cross-platform, aesthetic, distraction-free Markdown editor. It has built-in support for the Sundown processor, but can also auto-detect Pandoc, MultiMarkdown, Discount and cmark processors. It seeks to be an unobtrusive editor.

ghostwriter has a good feature set which includes syntax highlighting, a full-screen mode, a focus mode, themes, spell checking with Hunspell, a live word count, live HTML preview, and custom CSS style sheets for HTML preview, drag and drop support for images, and internalization support.  A Hemingway mode button disables backspace and delete keys. A new Markdown cheat sheet HUD window is a useful addition. Theme support is pretty basic, but there are some experimental themes available at this [GitHub repository][14].

ghostwriter is an under-rated utility. I have come to appreciate the versatility of this application more and more, in part because of its spartan interface helps the writer fully concentrate on curating content.  Recommended.

ghostwriter is available for Linux and Windows. There is also a portable version available for Windows.

Homepage: [https://github.com/wereturtle/ghostwriter][15]
License: GNU GPL v3

* * *

### Abricotine

 ![Abricotine - cross-platform Markdown editor](https://i2.wp.com/www.ossblog.org/wp-content/uploads/2017/02/Abricotine.png?resize=800%2C316&ssl=1) 

Abricotine is a promising cross-platform open-source markdown editor built for the desktop. It is available for Linux, OS X and Windows.

The application supports markdown syntax combined with some Github-flavored Markdown enhancements (such as tables). It lets users preview documents directly in the text editor as opposed to a side pane.

The tool has a reasonable set of features including a spell checker, the ability to save documents as HTML or copy rich text to paste in your email client. You can also display a document table of content in the side pane, display syntax highlighting for code, as well as helpers, anchors and hidden characters. It is at a fairly early stage of development with some basic bugs that need fixing, but it is one to keep an eye on. There are 2 themes, with the ability to add your own.

Homepage: [http://abricotine.brrd.fr/][16]
License: GNU General Public License v3 or later

* * *

### ReText

 ![ReText - Linux Markdown editor](https://i1.wp.com/www.ossblog.org/wp-content/uploads/2017/02/ReText.png?resize=800%2C270&ssl=1) 

ReText is a simple but powerful editor for Markdown and reStructuredText. It gives users the power to control all output formatting. The files it works with are plain text files, however it can export to PDF, HTML and other formats. ReText is officially supported on Linux only.

Features include:

*   Full screen mode
*   Live previews
*   Synchronised scrolling (for Markdown)
*   Support for math formulas
*   Spell checking
*   Page breaks
*   Export to HTML, ODT and PDF
*   Use other markup languages

Homepage: [https://github.com/retext-project/retext][17]
License: GNU GPL v2 or higher

--------------------------------------------------------------------------------

via: https://www.ossblog.org/markdown-editors/

作者：[Steve Emms ][a]
译者：[译者ID](https://github.com/译者ID)
校对：[校对者ID](https://github.com/校对者ID)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]:https://www.ossblog.org/author/steve/
[1]:https://www.ossblog.org/author/steve/
[2]:https://www.ossblog.org/markdown-editors/#comments
[3]:https://www.ossblog.org/category/utilities/
[4]:https://remarkableapp.github.io/
[5]:https://www.ossblog.org/top-software/2/
[6]:https://atom.io/packages/markdown-preview-enhanced
[7]:https://atom.io/packages/markdown-scroll-sync
[8]:https://atom.io/packages/markdown-writer
[9]:https://atom.io/packages/markdown-pdf
[10]:https://atom.io/
[11]:http://pad.haroopress.com/
[12]:https://stackedit.io/
[13]:https://macdown.uranusjr.com/
[14]:https://github.com/jggouvea/ghostwriter-themes
[15]:https://github.com/wereturtle/ghostwriter
[16]:http://abricotine.brrd.fr/
[17]:https://github.com/retext-project/retext
